@@ -40,8 +40,6 @@
 #include <kservice.h>
 
 #include "ksgrd/SensorManager.h"
-#include "TimerSettings.h"
-
 #include "SensorDisplay.h"
 
 #define NONE -1
@@ -87,6 +85,8 @@ SensorDisplay::~SensorDisplay()
 
   if ( mTimerId > 0 )
     killTimer( mTimerId );
+  for(int i = mSensors.size()-1; i>=0; i--)
+    unregisterSensor(i);
 }
 
 void SensorDisplay::registerSensor( SensorProperties *sp )
@@ -113,19 +113,13 @@ void SensorDisplay::showContextMenu(const QPoint &pos)
     QMenu pm;
     QAction *action = 0;
     bool menuEmpty = true;
-    if ( mSharedSettings->isApplet ) {
-      action = pm.addAction( i18n( "Launch &System Guard") );
-      action->setData( 1 );
-      pm.addSeparator();
-      menuEmpty = false;
-    }
 
     if ( hasSettingsDialog() ) {
       action = pm.addAction( i18n( "&Properties" ) );
       action->setData( 2 );
       menuEmpty = false;
     }
-    if(!mSharedSettings->locked) {  
+    if(mSharedSettings && !mSharedSettings->locked) {  
       action = pm.addAction( i18n( "&Remove Display" ) );
       action->setData( 3 );
       menuEmpty = false;
@@ -232,7 +226,13 @@ QColor SensorDisplay::restoreColor( QDomElement &element, const QString &attr,
 void SensorDisplay::saveColor( QDomElement &element, const QString &attr,
                                const QColor &color )
 {
-  element.setAttribute( attr, color.rgba() );
+  element.setAttribute( attr, "0x" + QString::number(color.rgba(),16) );
+}
+
+void SensorDisplay::saveColorAppend( QDomElement &element, const QString &attr,
+                               const QColor &color )
+{
+  element.setAttribute( attr, element.attribute(attr) + ",0x" + QString::number(color.rgba(),16) );
 }
 
 bool SensorDisplay::addSensor( const QString &hostName, const QString &name,
@@ -244,6 +244,8 @@ bool SensorDisplay::addSensor( const QString &hostName, const QString &name,
 
 bool SensorDisplay::removeSensor( uint pos )
 {
+  if(pos >= mSensors.count())
+    return false;
   unregisterSensor( pos );
   return true;
 }
@@ -320,7 +322,7 @@ void SensorDisplay::setSensorOk( bool ok )
     if ( !mPlotterWdg || mPlotterWdg->isVisible())
       return;
 
-    QPixmap errorIcon = KIconLoader::global()->loadIcon( "connect_creating", KIconLoader::Desktop,
+    QPixmap errorIcon = KIconLoader::global()->loadIcon( "dialog-error", KIconLoader::Desktop,
                                              KIconLoader::SizeSmall );
 
     mErrorIndicator = new QWidget( mPlotterWdg );
@@ -345,7 +347,11 @@ void SensorDisplay::changeEvent( QEvent * event ) {
 void SensorDisplay::setTitle( const QString &title )
 {
   mTitle = title;
-  mTranslatedTitle = i18n(title.toLatin1());
+  // Check if the string is latin1 only.  If so, try to translate
+  if(QString::fromLatin1(title.toLatin1()) == title)
+    mTranslatedTitle = i18n(title.toLatin1());
+  else
+    mTranslatedTitle = mTitle;
   emit titleChanged(mTitle);
   emit translatedTitleChanged(mTranslatedTitle);
 }
@@ -385,17 +391,6 @@ void SensorDisplay::setPlotterWidget( QWidget *wdg )
   mPlotterWdg = wdg;
 }
 
-void SensorDisplay::reorderSensors(const QList<int> &orderOfSensors)
-{
-  QList<SensorProperties *> newSensors;
-  for ( int i = 0; i < orderOfSensors.count(); ++i ) {
-    newSensors.append( mSensors.at(orderOfSensors[i] ));
-  }
-  mSensors = newSensors;
-}
-
-
-
 SensorProperties::SensorProperties()
 {
 }
@@ -478,4 +473,13 @@ bool SensorProperties::isOk() const
   return mOk;
 }
 
+void SensorProperties::setRegExpName( const QString &name )
+{
+  mRegExpName = name;
+}
+QString SensorProperties::regExpName() const
+{
+  return mRegExpName;
+}
+
 #include "SensorDisplay.moc"
