@@ -861,6 +861,10 @@ void Client::setActive( bool act )
     if ( active == act )
         return;
     active = act;
+    const int ruledOpacity = active
+        ? rules()->checkOpacityActive( qRound( opacity() * 100.0 ))
+        : rules()->checkOpacityInactive( qRound( opacity() * 100.0 ));
+    setOpacity( ruledOpacity/100.0 );
     workspace()->setActiveClient( act ? this : NULL, Allowed );
     
     if ( active )
