@@ -3,6 +3,7 @@
  This file is part of the KDE project.
 
 Copyright (C) 2006 Lubos Lunak <l.lunak@kde.org>
+Copyright (C) 2009 Lucas Murray <lmurray@undefinedfire.com>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -33,6 +34,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <kdebug.h>
 #include <ksharedconfig.h>
+#include <kstandarddirs.h>
 #include <kconfiggroup.h>
 
 #include <assert.h>
@@ -112,6 +114,11 @@ void Effect::reconfigure( ReconfigureFlags )
     {
     }
 
+const void* Effect::proxy() const
+    {
+    return NULL;
+    }
+
 void Effect::windowUserMovedResized( EffectWindow* , bool, bool )
     {
     }
@@ -953,7 +960,7 @@ QRect PaintClipper::Iterator::boundingRect() const
  TimeLine
 ***************************************************************/
 
-TimeLine::TimeLine(const int duration)
+TimeLine::TimeLine(int duration)
     {
     m_Time = 0;
     m_Duration = duration;
@@ -1291,6 +1298,18 @@ QRectF WindowMotionManager::transformedGeometry( EffectWindow *w ) const
     return geometry;
     }
 
+QRectF WindowMotionManager::targetGeometry( EffectWindow *w ) const
+    {
+    QRectF geometry( w->geometry() );
+
+    // TODO: Take into account existing scale so that we can work with multiple managers (E.g. Present windows + grid)
+    geometry.moveTo( m_managedWindows[ w ].translation.target() );
+    geometry.setWidth( geometry.width() * m_managedWindows[ w ].scale.target().x() );
+    geometry.setHeight( geometry.height() * m_managedWindows[ w ].scale.target().y() );
+
+    return geometry;
+    }
+
 EffectWindow* WindowMotionManager::windowAtPoint( QPoint point, bool useStackingOrder ) const
     {
     // TODO: Stacking order uses EffectsHandler::stackingOrder() then filters by m_managedWindows
@@ -1303,4 +1322,505 @@ EffectWindow* WindowMotionManager::windowAtPoint( QPoint point, bool useStacking
     return NULL;
     }
 
+/***************************************************************
+ EffectFrame
+***************************************************************/
+
+GLTexture* EffectFrame::m_unstyledTexture = NULL;
+
+EffectFrame::EffectFrame( Style style, bool staticSize, QPoint position, Qt::Alignment alignment )
+    : QObject()
+    , m_style( style )
+    , m_static( staticSize )
+    , m_point( position )
+    , m_alignment( alignment )
+    {
+#ifdef KWIN_HAVE_OPENGL_COMPOSITING
+    m_texture = NULL;
+    m_textTexture = NULL;
+#endif
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
+    m_picture = NULL;
+    m_textPicture = NULL;
+#endif
+
+    if( m_style == Unstyled )
+        {
+#ifdef KWIN_HAVE_OPENGL_COMPOSITING
+        if( effects->compositingType() == OpenGLCompositing && !m_unstyledTexture )
+            updateUnstyledTexture();
+#endif
+        }
+    else if( m_style == Styled )
+        {
+        m_frame.setImagePath( "widgets/background" );
+        m_frame.setCacheAllRenderedFrames( true );
+        connect( Plasma::Theme::defaultTheme(), SIGNAL( themeChanged() ), this, SLOT( plasmaThemeChanged() ));
+        }
+    }
+
+EffectFrame::~EffectFrame()
+    {
+#ifdef KWIN_HAVE_OPENGL_COMPOSITING
+    delete m_texture;
+    delete m_textTexture;
+#endif
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
+    delete m_picture;
+    delete m_textPicture;
+#endif
+    }
+
+void EffectFrame::free()
+    {
+#ifdef KWIN_HAVE_OPENGL_COMPOSITING
+    delete m_texture;
+    m_texture = NULL;
+    delete m_textTexture;
+    m_textTexture = NULL;
+#endif
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
+    delete m_picture;
+    m_picture = NULL;
+    delete m_textPicture;
+    m_textPicture = NULL;
+#endif
+    }
+
+void EffectFrame::render( QRegion region, double opacity, double frameOpacity )
+    {
+    if( m_geometry.isEmpty() )
+        return; // Nothing to display
+
+    region = infiniteRegion(); // TODO: Old region doesn't seem to work with OpenGL
+
+#ifdef KWIN_HAVE_OPENGL_COMPOSITING
+    if( effects->compositingType() == OpenGLCompositing )
+        {
+        glPushAttrib( GL_CURRENT_BIT | GL_ENABLE_BIT | GL_TEXTURE_BIT );
+        glEnable( GL_BLEND );
+        glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
+        glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
+
+        glPushMatrix();
+
+        // Render the actual frame
+        if( m_style == Unstyled )
+            {
+            const QRect& area = m_geometry.adjusted( -5, -5, 5, 5 );
+            const int roundness = 5;
+            QVector<float> verts, texCoords;
+            verts.reserve( 80 );
+            texCoords.reserve( 80 );
+
+            // Center
+            addQuadVertices( verts, area.left() + roundness, area.top() + roundness,
+                area.right() - roundness, area.bottom() - roundness );
+            addQuadVertices( texCoords, 0.5, 0.5, 0.5, 0.5 );
+
+            // Left
+            addQuadVertices( verts, area.left(), area.top() + roundness,
+                area.left() + roundness, area.bottom() - roundness );
+            addQuadVertices( texCoords, 0.0, 0.5, 0.5, 0.5 );
+            // Top
+            addQuadVertices( verts, area.left() + roundness, area.top(),
+                area.right() - roundness, area.top() + roundness );
+            addQuadVertices( texCoords, 0.5, 0.0, 0.5, 0.5 );
+            // Right
+            addQuadVertices( verts, area.right() - roundness, area.top() + roundness,
+                area.right(), area.bottom() - roundness );
+            addQuadVertices( texCoords, 0.5, 0.5, 1.0, 0.5 );
+            // Bottom
+            addQuadVertices( verts, area.left() + roundness, area.bottom() - roundness,
+                area.right() - roundness, area.bottom() );
+            addQuadVertices( texCoords, 0.5, 0.5, 0.5, 1.0 );
+
+            // Top-left
+            addQuadVertices( verts, area.left(), area.top(),
+                area.left() + roundness, area.top() + roundness );
+            addQuadVertices( texCoords, 0.0, 0.0, 0.5, 0.5 );
+            // Top-right
+            addQuadVertices( verts, area.right() - roundness, area.top(),
+                area.right(), area.top() + roundness );
+            addQuadVertices( texCoords, 0.5, 0.0, 1.0, 0.5 );
+            // Bottom-left
+            addQuadVertices( verts, area.left(), area.bottom() - roundness,
+                area.left() + roundness, area.bottom() );
+            addQuadVertices( texCoords, 0.0, 0.5, 0.5, 1.0 );
+            // Bottom-right
+            addQuadVertices( verts, area.right() - roundness, area.bottom() - roundness,
+                area.right(), area.bottom() );
+            addQuadVertices( texCoords, 0.5, 0.5, 1.0, 1.0 );
+
+            glColor4f( 0.0, 0.0, 0.0, opacity * frameOpacity );
+
+            m_unstyledTexture->bind();
+            m_unstyledTexture->enableNormalizedTexCoords();
+            renderGLGeometry( verts.count() / 2, verts.data(), texCoords.data() );
+            m_unstyledTexture->disableNormalizedTexCoords();
+            m_unstyledTexture->unbind();
+            }
+        else if( m_style == Styled )
+            {
+            if( !m_texture ) // Lazy creation
+                updateTexture();
+
+            glColor4f( 1.0, 1.0, 1.0, opacity * frameOpacity );
+
+            m_texture->bind();
+            qreal left, top, right, bottom;
+            m_frame.getMargins( left, top, right, bottom ); // m_geometry is the inner geometry
+            m_texture->render( region, m_geometry.adjusted( -left, -top, right, bottom ));
+            m_texture->unbind();
+            }
+
+        glColor4f( 1.0, 1.0, 1.0, opacity );
+
+        // Render icon
+        if( !m_icon.isNull() && !m_iconSize.isEmpty() )
+            {
+            QPoint topLeft( m_geometry.x(), m_geometry.center().y() - m_iconSize.height() / 2 );
+
+            GLTexture* icon = new GLTexture( m_icon ); // TODO: Cache
+            icon->bind();
+            icon->render( region, QRect( topLeft, m_iconSize ));
+            icon->unbind();
+            delete icon;
+            }
+
+        // Render text
+        if( !m_text.isEmpty() )
+            {
+            if( !m_textTexture ) // Lazy creation
+                updateTextTexture();
+            m_textTexture->bind();
+            m_textTexture->render( region, m_geometry );
+            m_textTexture->unbind();
+            }
+
+        glPopMatrix();
+        glPopAttrib();
+        }
+#endif
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
+    if( effects->compositingType() == XRenderCompositing )
+        {
+        // Render the actual frame
+        if( m_style == Unstyled )
+            xRenderRoundBox( effects->xrenderBufferPicture(), m_geometry.adjusted( -5, -5, 5, 5 ),
+                5, QColor( 0, 0, 0, int( opacity * frameOpacity * 255 )));
+        else if( m_style == Styled )
+            {
+            if( !m_picture ) // Lazy creation
+                updatePicture();
+            qreal left, top, right, bottom;
+            m_frame.getMargins( left, top, right, bottom ); // m_geometry is the inner geometry
+            QRect geom = m_geometry.adjusted( -left, -top, right, bottom );
+            XRenderComposite( display(), PictOpOver, *m_picture, None, effects->xrenderBufferPicture(),
+                0, 0, 0, 0, geom.x(), geom.y(), geom.width(), geom.height() );
+            }
+
+        XRenderPicture fill = xRenderBlendPicture(opacity);
+        
+        // Render icon
+        if( !m_icon.isNull() && !m_iconSize.isEmpty() )
+            {
+            QPoint topLeft( m_geometry.x(), m_geometry.center().y() - m_iconSize.height() / 2 );
+
+            XRenderPicture* icon = new XRenderPicture( m_icon ); // TODO: Cache
+            QRect geom = QRect( topLeft, m_iconSize );
+            XRenderComposite( display(), PictOpOver, *icon, fill, effects->xrenderBufferPicture(),
+                0, 0, 0, 0, geom.x(), geom.y(), geom.width(), geom.height() );
+            delete icon;
+            }
+
+        // Render text
+        if( !m_text.isEmpty() )
+            {
+            if( !m_textPicture ) // Lazy creation
+                updateTextPicture();
+            XRenderComposite( display(), PictOpOver, *m_textPicture, fill, effects->xrenderBufferPicture(),
+                0, 0, 0, 0, m_geometry.x(), m_geometry.y(), m_geometry.width(), m_geometry.height() );
+            }
+        }
+#endif
+    }
+
+void EffectFrame::setPosition( const QPoint& point )
+    {
+    m_point = point;
+    autoResize();
+    }
+
+void EffectFrame::setGeometry( const QRect& geometry, bool force )
+    {
+    QRect oldGeom = m_geometry;
+    m_geometry = geometry;
+    if( m_geometry == oldGeom && !force )
+        return;
+    effects->addRepaint( oldGeom );
+    effects->addRepaint( m_geometry );
+    if( m_geometry.size() == oldGeom.size() && !force )
+        return;
+
+    if( m_style == Styled )
+        {
+        qreal left, top, right, bottom;
+        m_frame.getMargins( left, top, right, bottom ); // m_geometry is the inner geometry
+        m_frame.resizeFrame( m_geometry.adjusted( -left, -top, right, bottom ).size() );
+        }
+
+#ifdef KWIN_HAVE_OPENGL_COMPOSITING
+    if( effects->compositingType() == OpenGLCompositing )
+        {
+        delete m_texture;
+        m_texture = NULL;
+        delete m_textTexture;
+        m_textTexture = NULL;
+        }
+#endif
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
+    if( effects->compositingType() == XRenderCompositing )
+        {
+        delete m_picture;
+        m_picture = NULL;
+        delete m_textPicture;
+        m_textPicture = NULL;
+        }
+#endif
+    }
+
+void EffectFrame::setText( const QString& text )
+    {
+    m_text = text;
+    QRect oldGeom = m_geometry;
+    autoResize();
+    if( oldGeom == m_geometry )
+        { // Wasn't updated in autoResize()
+#ifdef KWIN_HAVE_OPENGL_COMPOSITING
+        if( effects->compositingType() == OpenGLCompositing )
+            {
+            delete m_textTexture;
+            m_textTexture = NULL;
+            }
+#endif
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
+        if( effects->compositingType() == XRenderCompositing )
+            {
+            delete m_textPicture;
+            m_textPicture = NULL;
+            }
+#endif
+        }
+    }
+
+void EffectFrame::setFont( const QFont& font )
+    {
+    m_font = font;
+    QRect oldGeom = m_geometry;
+    if( !m_text.isEmpty() )
+        autoResize();
+    if( oldGeom == m_geometry )
+        { // Wasn't updated in autoResize()
+#ifdef KWIN_HAVE_OPENGL_COMPOSITING
+        if( effects->compositingType() == OpenGLCompositing )
+            {
+            delete m_textTexture;
+            m_textTexture = NULL;
+            }
+#endif
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
+        if( effects->compositingType() == XRenderCompositing )
+            {
+            delete m_textPicture;
+            m_textPicture = NULL;
+            }
+#endif
+        }
+    }
+
+void EffectFrame::setIcon( const QPixmap& icon )
+    {
+    m_icon = icon;
+    if( m_iconSize.isEmpty() ) // Set a size if we don't already have one
+        setIconSize( m_icon.size() );
+    }
+
+void EffectFrame::setIconSize( const QSize& size )
+    {
+    m_iconSize = size;
+    autoResize();
+    }
+
+QColor EffectFrame::styledTextColor()
+    {
+    return Plasma::Theme::defaultTheme()->color( Plasma::Theme::TextColor );
+    }
+
+void EffectFrame::plasmaThemeChanged()
+    {
+#ifdef KWIN_HAVE_OPENGL_COMPOSITING
+    if( effects->compositingType() == OpenGLCompositing )
+        {
+        delete m_texture;
+        m_texture = NULL;
+        delete m_textTexture;
+        m_textTexture = NULL;
+        }
+#endif
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
+    if( effects->compositingType() == XRenderCompositing )
+        {
+        delete m_picture;
+        m_picture = NULL;
+        delete m_textPicture;
+        m_textPicture = NULL;
+        }
+#endif
+    }
+
+void EffectFrame::autoResize()
+    {
+    if( m_static )
+        return; // Not automatically resizing
+
+    QRect geometry;
+
+    // Set size
+    if( !m_text.isEmpty() )
+        {
+        QFontMetrics metrics( m_font );
+        geometry.setSize( metrics.size( 0, m_text ));
+        }
+    if( !m_icon.isNull() && !m_iconSize.isEmpty() )
+        {
+        geometry.setLeft( -m_iconSize.width() );
+        if( m_iconSize.height() > geometry.height() )
+            geometry.setHeight( m_iconSize.height() );
+        }
+
+    // Set position
+    if( m_alignment & Qt::AlignLeft )
+        geometry.moveLeft( m_point.x() );
+    else if( m_alignment & Qt::AlignRight )
+        geometry.moveLeft( m_point.x() - geometry.width() );
+    else
+        geometry.moveLeft( m_point.x() - geometry.width() / 2 );
+    if( m_alignment & Qt::AlignTop )
+        geometry.moveTop( m_point.y() );
+    else if( m_alignment & Qt::AlignBottom )
+        geometry.moveTop( m_point.y() - geometry.height() );
+    else
+        geometry.moveTop( m_point.y() - geometry.height() / 2 );
+
+    setGeometry( geometry );
+    }
+
+void EffectFrame::updateTexture()
+    {
+#ifdef KWIN_HAVE_OPENGL_COMPOSITING
+    delete m_texture;
+    if( m_style == Styled )
+        m_texture = new GLTexture( m_frame.framePixmap() );
+#endif
+    }
+
+void EffectFrame::updateTextTexture()
+    {
+#ifdef KWIN_HAVE_OPENGL_COMPOSITING
+    delete m_textTexture;
+
+    if( m_text.isEmpty() )
+        return;
+
+    // Determine position on texture to paint text
+    QRect rect( QPoint( 0, 0 ), m_geometry.size() );
+    if( !m_icon.isNull() && !m_iconSize.isEmpty() )
+        rect.setLeft( m_iconSize.width() );
+
+    // If static size elide text as required
+    QString text = m_text;
+    if( m_static )
+        {
+        QFontMetrics metrics( m_font );
+        text = metrics.elidedText( text, Qt::ElideRight, rect.width() );
+        }
+
+    QImage image( m_geometry.size(), QImage::Format_ARGB32 );
+    image.fill( Qt::transparent );
+    QPainter p( &image );
+    p.setFont( m_font );
+    if( m_style == Styled )
+        p.setPen( styledTextColor() );
+    else // TODO: What about no frame? Custom color setting required
+        p.setPen( Qt::white );
+    p.drawText( rect, m_alignment, text );
+    p.end();
+    m_textTexture = new GLTexture( image );
+#endif
+    }
+
+void EffectFrame::updatePicture()
+    {
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
+    delete m_picture;
+    if( m_style == Styled )
+        m_picture = new XRenderPicture( m_frame.framePixmap() );
+#endif
+    }
+
+void EffectFrame::updateTextPicture()
+    { // Mostly copied from EffectFrame::updateTextTexture() above
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
+    delete m_textPicture;
+
+    if( m_text.isEmpty() )
+        return;
+
+    // Determine position on texture to paint text
+    QRect rect( QPoint( 0, 0 ), m_geometry.size() );
+    if( !m_icon.isNull() && !m_iconSize.isEmpty() )
+        rect.setLeft( m_iconSize.width() );
+
+    // If static size elide text as required
+    QString text = m_text;
+    if( m_static )
+        {
+        QFontMetrics metrics( m_font );
+        text = metrics.elidedText( text, Qt::ElideRight, rect.width() );
+        }
+
+    QPixmap pixmap( m_geometry.size() );
+    pixmap.fill( Qt::transparent );
+    QPainter p( &pixmap );
+    p.setFont( m_font );
+    if( m_style == Styled )
+        p.setPen( styledTextColor() );
+    else // TODO: What about no frame? Custom color setting required
+        p.setPen( Qt::white );
+    p.drawText( rect, m_alignment, text );
+    p.end();
+    m_textPicture = new XRenderPicture( pixmap );
+#endif
+    }
+
+void EffectFrame::updateUnstyledTexture()
+    {
+#ifdef KWIN_HAVE_OPENGL_COMPOSITING
+    delete m_unstyledTexture;
+    // Based off circle() from kwinxrenderutils.cpp
+#define CS 8
+    QImage tmp( 2 * CS, 2 * CS, QImage::Format_ARGB32 );
+    tmp.fill( Qt::transparent );
+    QPainter p( &tmp );
+    p.setRenderHint( QPainter::Antialiasing );
+    p.setPen( Qt::NoPen );
+    p.setBrush( Qt::black );
+    p.drawEllipse( tmp.rect() );
+    p.end();
+#undef CS
+    m_unstyledTexture = new GLTexture( tmp );
+#endif
+    }
+
 } // namespace
