@@ -57,8 +57,28 @@ namespace KWin
 
 #ifndef KCMRULES
 
-void Motif::readFlags( WId w, bool& noborder, bool& resize, bool& move,
-    bool& minimize, bool& maximize, bool& close )
+//************************************
+// StrutRect
+//************************************
+
+StrutRect::StrutRect( QRect rect, StrutArea area )
+    : QRect( rect )
+    , m_area( area )
+    {
+    }
+
+StrutRect::StrutRect( const StrutRect& other )
+    : QRect( other )
+    , m_area( other.area() )
+    {
+    }
+
+//************************************
+// Motif
+//************************************
+
+void Motif::readFlags( WId w, bool& got_noborder, bool& noborder,
+    bool& resize, bool& move, bool& minimize, bool& maximize, bool& close )
     {
     Atom type;
     int format;
@@ -72,6 +92,7 @@ void Motif::readFlags( WId w, bool& noborder, bool& resize, bool& move,
         if ( data )
             hints = (MwmHints*) data;
         }
+    got_noborder = false;
     noborder = false;
     resize = true;
     move = true;
@@ -97,10 +118,10 @@ void Motif::readFlags( WId w, bool& noborder, bool& resize, bool& move,
             if( hints->functions & MWM_FUNC_CLOSE )
                 close = set_value;
             }
-        if ( hints->flags & MWM_HINTS_DECORATIONS ) 
+        if ( hints->flags & MWM_HINTS_DECORATIONS )
             {
-            if ( hints->decorations == 0 )
-                noborder = true;
+            got_noborder = true;
+            noborder = !hints->decorations;
             }
         XFree( data );
         }
@@ -334,6 +355,10 @@ Qt::MouseButton x11ToQtMouseButton( int button )
         return Qt::MidButton;
     if( button == Button3 )
         return Qt::RightButton;
+    if( button == Button4 )
+        return Qt::XButton1;
+    if( button == Button5 )
+        return Qt::XButton2;
     return Qt::NoButton;
     }
 
@@ -366,6 +391,10 @@ Qt::MouseButtons x11ToQtMouseButtons( int state )
         ret |= Qt::MidButton;
     if( state & Button3Mask )
         ret |= Qt::RightButton;
+    if( state & Button4Mask )
+        ret |= Qt::XButton1;
+    if( state & Button5Mask )
+        ret |= Qt::XButton2;
     return ret;
     }
 
