@@ -162,7 +162,7 @@ void NotifierDialog::insertDevice(const QString &name)
     m_notifierView->calculateRects();
 }
 
-void NotifierDialog::setUnMount(bool unmount, const QString &name) 
+void NotifierDialog::setUnMount(bool unmount, const QString &name)
 {
     QModelIndex index = indexForUdi(name);
     if (!index.isValid()) {
@@ -249,14 +249,18 @@ QString NotifierDialog::getDeviceUdi(int index)
 void NotifierDialog::buildDialog()
 {
     m_widget = new QWidget();
+    m_widget->setAttribute(Qt::WA_TranslucentBackground);
+    QPalette p = m_widget->palette();
+    p.setColor(QPalette::Window, Qt::transparent);
+    m_widget->setPalette(p);
 
     QVBoxLayout *l_layout = new QVBoxLayout(m_widget);
     l_layout->setSpacing(0);
     l_layout->setMargin(0);
-    
+
     m_label = new QLabel(m_widget);
     updateColors();
-    
+
     QLabel *icon = new QLabel(m_widget);
     icon->setPixmap(KIcon("emblem-mounted").pixmap(KIconLoader::SizeMedium, KIconLoader::SizeMedium));
 
@@ -411,9 +415,6 @@ void NotifierDialog::updateColors()
     KColorScheme colorTheme = KColorScheme(QPalette::Active, KColorScheme::View,Plasma::Theme::defaultTheme()->colorScheme());
     m_label->setText(i18n("<font color=\"%1\">Devices recently plugged in:</font>",colorTheme.foreground(KColorScheme::NormalText).color().name()));
 
-    QPalette p = m_widget->palette();
-    p.setColor(QPalette::Window, Plasma::Theme::defaultTheme()->color(Plasma::Theme::BackgroundColor));
-    m_widget->setPalette(p);
 }
 
 #include "notifierdialog.moc"
