@@ -161,7 +161,9 @@ void NotificationWidgetPrivate::setTextFields(const QString &applicationName,
         extenderItem->setTitle(i18n("Notification from %1", applicationName));
     }
 
-    body->setHtml(message);
+    QString processed = message.trimmed();
+    processed.replace("\n", "<br>");
+    body->setHtml(processed);
 }
 
 void NotificationWidgetPrivate::completeDetach()
@@ -226,15 +228,15 @@ void NotificationWidgetPrivate::updateNotification()
         imageHeight = notification->image().size().height();
     }
 
+    extenderItem->showCloseButton();
+
     //set the correct size hint and display a close action if no actions are provided by the
     //notification
     qreal bodyHeight = body->boundingRect().height();
     if (actionsWidget) {
-        extenderItem->hideCloseButton();
         q->setPreferredWidth(qMax(q->preferredWidth(), actionsWidget->preferredWidth()));
         q->setPreferredHeight(qMax(bodyHeight + actionsWidget->size().height(), imageHeight));
     } else {
-        extenderItem->showCloseButton();
         q->setPreferredHeight(qMax(bodyHeight, imageHeight));
     }
 }
