@@ -51,6 +51,10 @@ Tasks::Tasks(QObject* parent, const QVariantList &arguments)
        m_topMargin(0),
        m_rightMargin(0),
        m_bottomMargin(0),
+       m_offscreenLeftMargin(0),
+       m_offscreenTopMargin(0),
+       m_offscreenRightMargin(0),
+       m_offscreenBottomMargin(0),
        m_rootGroupItem(0),
        m_groupManager(0),
        m_groupModifierKey(Qt::AltModifier)
@@ -76,6 +80,9 @@ void Tasks::init()
 {
     //kDebug();
 
+    const KConfigGroup cg = config();
+    m_showTooltip = cg.readEntry("showTooltip", true);
+
     m_groupManager = new TaskManager::GroupManager(this);
     Plasma::Containment* appletContainment = containment();
     if (appletContainment) {
@@ -117,13 +124,10 @@ void Tasks::init()
 
     setLayout(layout);
 
-    KConfigGroup cg = config();
-
     m_groupManager->setShowOnlyCurrentDesktop( cg.readEntry("showOnlyCurrentDesktop", false));
     m_groupManager->setShowOnlyCurrentScreen( cg.readEntry("showOnlyCurrentScreen", false));
     m_groupManager->setShowOnlyMinimized( cg.readEntry("showOnlyMinimized", false));
     m_groupManager->setOnlyGroupWhenFull(cg.readEntry("groupWhenFull", true));
-    m_showTooltip = cg.readEntry("showTooltip", true);
 
     m_groupManager->setGroupingStrategy( static_cast<TaskManager::GroupManager::TaskGroupingStrategy>(cg.readEntry("groupingStrategy", static_cast<int>(TaskManager::GroupManager::ProgramGrouping))));
 
