@@ -35,7 +35,6 @@ class QAction;
 class KCModuleProxy;
 class KFilePlacesModel;
 class KProcess;
-class KPropertiesDialog;
 
 namespace Plasma
 {
@@ -58,6 +57,8 @@ class Trash : public Plasma::Applet
         void slotEmpty();
 
     protected:
+        void dragEnterEvent(QGraphicsSceneDragDropEvent *event);
+        void dragLeaveEvent(QGraphicsSceneDragDropEvent *event);
         void dropEvent(QGraphicsSceneDragDropEvent *event);
         void createConfigurationInterface(KConfigDialog *parent);
         void createMenu();
