@@ -44,101 +44,53 @@
 #include <QApplication>
 #include <QDesktopWidget>
 #include <QPixmapCache>
-#include <QTimer>
 #include <QtDBus/QtDBus>
+#include <QTimer>
 
 #include <KAction>
 #include <KCrash>
 #include <KDebug>
 #include <KCmdLineArgs>
+#include <KGlobalAccel>
 #include <KWindowSystem>
 
 #include <ksmserver_interface.h>
 
 #include <Plasma/Containment>
 #include <Plasma/Theme>
+#include <Plasma/Dialog>
+
+#include <kephal/screens.h>
 
 #include "appletbrowser.h"
 #include "appadaptor.h"
 #include "backgrounddialog.h"
+#include "checkbox.h"
 #include "desktopcorona.h"
 #include "desktopview.h"
 #include "panelview.h"
 #include "plasma-shell-desktop.h"
-
-#include <kephal/screens.h>
+#include "toolbutton.h"
 
 #ifdef Q_WS_X11
 #include <X11/Xlib.h>
 #include <X11/extensions/Xrender.h>
-
-Display* dpy = 0;
-Colormap colormap = 0;
-Visual *visual = 0;
 #endif
 
-void checkComposite()
-{
-#ifdef Q_WS_X11
-    dpy = XOpenDisplay(0); // open default display
-    if (!dpy) {
-        kError() << "Cannot connect to the X server" << endl;
-        return;
-    }
-    if( qgetenv( "KDE_SKIP_ARGB_VISUALS" ) == "1" )
-        return;
-
-    int screen = DefaultScreen(dpy);
-    int eventBase, errorBase;
-
-    if (XRenderQueryExtension(dpy, &eventBase, &errorBase)) {
-        int nvi;
-        XVisualInfo templ;
-        templ.screen  = screen;
-        templ.depth   = 32;
-        templ.c_class = TrueColor;
-        XVisualInfo *xvi = XGetVisualInfo(dpy,
-                                          VisualScreenMask | VisualDepthMask | VisualClassMask,
-                                          &templ, &nvi);
-        for (int i = 0; i < nvi; ++i) {
-            XRenderPictFormat *format = XRenderFindVisualFormat(dpy, xvi[i].visual);
-            if (format->type == PictTypeDirect && format->direct.alphaMask) {
-                visual = xvi[i].visual;
-                colormap = XCreateColormap(dpy, RootWindow(dpy, screen), visual, AllocNone);
-                break;
-            }
-        }
-        XFree(xvi);
-    }
-
-    kDebug() << (colormap ? "Plasma has an argb visual" : "Plasma lacks an argb visual") << visual << colormap;
-    kDebug() << ((KWindowSystem::compositingActive() && colormap) ? "Plasma can use COMPOSITE for effects"
-                                                                    : "Plasma is COMPOSITE-less") << "on" << dpy;
-#endif
-}
-
 PlasmaApp* PlasmaApp::self()
 {
     if (!kapp) {
-        checkComposite();
-#ifdef Q_WS_X11
-        return new PlasmaApp(dpy, visual ? Qt::HANDLE(visual) : 0, colormap ? Qt::HANDLE(colormap) : 0);
-#else
-        return new PlasmaApp(0, 0, 0);
-#endif
+        return new PlasmaApp();
     }
 
     return qobject_cast<PlasmaApp*>(kapp);
 }
 
-PlasmaApp::PlasmaApp(Display* display, Qt::HANDLE visual, Qt::HANDLE colormap)
-#ifdef Q_WS_X11
-    : KUniqueApplication(display, visual, colormap),
-#else
+PlasmaApp::PlasmaApp()
     : KUniqueApplication(),
-#endif
       m_corona(0),
       m_appletBrowser(0),
+      m_controllerDialog(0),
       m_zoomLevel(Plasma::DesktopZoom),
       m_panelHidden(0)
 {
@@ -146,6 +98,51 @@ PlasmaApp::PlasmaApp(Display* display, Qt::HANDLE visual, Qt::HANDLE colormap)
     KGlobal::locale()->insertCatalog("plasma-shells-common");
     KCrash::setFlags(KCrash::AutoRestart);
 
+    // why is the next line of code here here?
+    //
+    // plasma-desktop was once plasma. not a big deal, right?
+    // 
+    // well, kglobalaccel has a policy of forever
+    // reserving shortcuts. even if the application is not running, it will still
+    // defend that application's right to using that global shortcut. this has,
+    // at least to me, some very obvious negative impacts on usability, such as
+    // making it difficult for the user to switch between applications of the 
+    // same type and use the same global shortcuts, or when the component changes
+    // name as in plasma-desktop.
+    //
+    // applications can unregister each other's shortcuts, though, and so that's
+    // exactly what we do here.
+    //
+    // i'd love to just rely on the kconf_update script, but kglobalaccel doesn't
+    // listen to changes in its config file nor has a dbus interace to tickle it
+    // into re-reading it and it starts too early in the start up sequence for
+    // kconf_update to beat it to the config file.
+    //
+    // so we instead deal with a dbus roundtrip with kded 
+    // (8 context switches at minimum iirc?)
+    // at every app start for something that really only needs to be done once
+    // but which we can't know for sure when it has been done.
+    //
+    // if kglobalaccel ignored non-running apps or could be prompted into
+    // reloading its config, this would be unecessary.
+    //
+    // what's kind of funny is that if plasma actually relied on kglobalaccel for
+    // the plasmoid shortcuts (which it can't because layouts change too often and
+    // can be stored/retreived making kglobalaccel management a non-option) this
+    // would be a total disaster. As it is it's "just" potentially losing the user's
+    // customization of the Ctrl+F12 default shortcut to bring up the dashboard.
+    //
+    // this line should be removed when we decide that 4.[012]->4.<current> is
+    // no longer a supported upgrade path. sometime after dragons make a comeback
+    // and can be seen circling the skies again.
+    // - aseigo.
+    KGlobalAccel::cleanComponent("plasma");
+
+    m_panelViewCreationTimer = new QTimer(this);
+    m_panelViewCreationTimer->setSingleShot(true);
+    m_panelViewCreationTimer->setInterval(0);
+    connect(m_panelViewCreationTimer, SIGNAL(timeout()), this, SLOT(createWaitingPanels()));
+
     new PlasmaAppAdaptor(this);
     QDBusConnection::sessionBus().registerObject("/App", this);
     notifyStartup(false);
@@ -215,14 +212,14 @@ PlasmaApp::PlasmaApp(Display* display, Qt::HANDLE visual, Qt::HANDLE colormap)
     kDebug() << "Setting the pixmap cache size to" << cacheSize << "kilobytes";
     QPixmapCache::setCacheLimit(cacheSize);
 
-    //TODO: Make the shortcut configurable
-    KAction *showAction = new KAction( this );
-    showAction->setText( i18n( "Show Dashboard" ) );
-    showAction->setObjectName( "Show Dashboard" ); // NO I18N
-    showAction->setGlobalShortcut( KShortcut( Qt::CTRL + Qt::Key_F12 ) );
-    connect( showAction, SIGNAL( triggered() ), this, SLOT( toggleDashboard() ) );
+    KAction *showAction = new KAction(this);
+    showAction->setText(i18n("Show Dashboard"));
+    showAction->setObjectName("Show Dashboard"); // NO I18N
+    showAction->setGlobalShortcut(KShortcut(Qt::CTRL + Qt::Key_F12));
+    connect(showAction, SIGNAL(triggered()), this, SLOT(toggleDashboard()));
 
-    connect(this, SIGNAL(aboutToQuit()), this, SLOT(cleanup()));
+    KGlobal::setAllowQuit(true);
+    KGlobal::ref();
     QTimer::singleShot(0, this, SLOT(setupDesktop()));
 }
 
@@ -235,7 +232,7 @@ void PlasmaApp::setupDesktop()
 {
 #ifdef Q_WS_X11
     Atom atoms[5];
-    const char *atomNames[] = {"XdndAware", "XdndEnter", "XdndFinished", "XdndPosition", "XdndStatus"};
+    const char * const atomNames[] = {"XdndAware", "XdndEnter", "XdndFinished", "XdndPosition", "XdndStatus"};
     XInternAtoms(QX11Info::display(), const_cast<char **>(atomNames), 5, False, atoms);
     m_XdndAwareAtom = atoms[0];
     m_XdndEnterAtom = atoms[1];
@@ -257,8 +254,25 @@ void PlasmaApp::setupDesktop()
     Kephal::Screens *screens = Kephal::Screens::self();
     connect(screens, SIGNAL(screenRemoved(int)), SLOT(screenRemoved(int)));
 
+    if (AppSettings::perVirtualDesktopViews()) {
+        connect(KWindowSystem::self(), SIGNAL(numberOfDesktopsChanged(int)),
+                this, SLOT(checkVirtualDesktopViews(int)));
+    }
+
+    // free the memory possibly occupied by the background image of the
+    // root window - login managers will typically set one
+    QPalette palette;
+    palette.setColor(desktop()->backgroundRole(), Qt::black);
+    desktop()->setPalette(palette);
+
     // and now, let everyone know we're ready!
     notifyStartup(true);
+    connect(this, SIGNAL(aboutToQuit()), this, SLOT(cleanup()));
+}
+
+void PlasmaApp::quit()
+{
+    KGlobal::deref();
 }
 
 void PlasmaApp::cleanup()
@@ -292,10 +306,6 @@ void PlasmaApp::cleanup()
     m_panels.clear();
     qDeleteAll(panels);
 
-    QHash<Plasma::Containment *, BackgroundDialog *> dialogs = m_configDialogs;
-    m_configDialogs.clear();
-    qDeleteAll(dialogs);
-
     delete m_corona;
 
     //TODO: This manual sync() should not be necessary. Remove it when
@@ -308,6 +318,33 @@ void PlasmaApp::syncConfig()
     KGlobal::config()->sync();
 }
 
+void PlasmaApp::toggleDashboardIfWindows()
+{
+    if (m_desktops.isEmpty()) {
+        return;
+    }
+
+    if (m_desktops.first()->isDashboardVisible()) {
+        toggleDashboard();
+        return;
+    }
+
+    const int desktop = KWindowSystem::currentDesktop();
+
+    foreach (WId id, KWindowSystem::stackingOrder()) {
+        const KWindowInfo info = KWindowSystem::windowInfo(id, NET::WMDesktop | NET::WMState |
+                                                               NET::XAWMState | NET::WMWindowType |
+                                                               NET::WMVisibleName);
+        NET::WindowType type = info.windowType(NET::Normal | NET::Dialog);
+        if ((type == NET::Normal || type == NET::Dialog) &&
+            info.isOnDesktop(desktop) && !(info.state() & NET::Hidden) /*!info.isMinimized()*/) {
+            kDebug() << info.visibleName() << info.state() << info.windowType(NET::Normal | NET::Dialog);
+            toggleDashboard();
+            return;
+        }
+    }
+}
+
 void PlasmaApp::toggleDashboard()
 {
     int currentScreen = 0;
@@ -320,7 +357,7 @@ void PlasmaApp::toggleDashboard()
         currentDesktop = KWindowSystem::currentDesktop();
     }
 
-    DesktopView *view = viewForScreen(currentScreen, currentDesktop);
+    DesktopView *view = viewForScreen(currentScreen, currentDesktop-1);
     if (!view) {
         kWarning() << "we don't have a DesktopView for the current screen!" << currentScreen << currentDesktop;
         return;
@@ -470,19 +507,16 @@ void PlasmaApp::screenRemoved(int id)
         }
     }
 
-    /*
-    TODO: remove panels when screen goes away.
-          first, however, we need to be able to reserve and restore the panelsettings
-          even when the view itself goes away
-    QMutableListIterator<PanelView*> it(m_panels);
-    while (it.hasNext()) {
-        PanelView *panel = it.next();
-        if (panel->screen() == i) {
+    QMutableListIterator<PanelView*> pIt(m_panels);
+    while (pIt.hasNext()) {
+        PanelView *panel = pIt.next();
+        if (panel->screen() == id) {
+            kDebug() << "removing a panel for screen" << id;
+            panel->setContainment(0);
+            pIt.remove();
             delete panel;
-            it.remove();
         }
     }
-    */
 }
 
 DesktopView* PlasmaApp::viewForScreen(int screen, int desktop) const
@@ -506,12 +540,35 @@ Plasma::Corona* PlasmaApp::corona()
         connect(c, SIGNAL(containmentAdded(Plasma::Containment*)),
                 this, SLOT(containmentAdded(Plasma::Containment*)));
         connect(c, SIGNAL(configSynced()), this, SLOT(syncConfig()));
+        connect(c, SIGNAL(immutabilityChanged(Plasma::ImmutabilityType)),
+                this, SLOT(updateActions(Plasma::ImmutabilityType)));
 
         foreach (DesktopView *view, m_desktops) {
             connect(c, SIGNAL(screenOwnerChanged(int,int,Plasma::Containment*)),
                     view, SLOT(screenOwnerChanged(int,int,Plasma::Containment*)));
         }
 
+        //actions!
+        KAction *activityAction = c->addAction("add sibling containment");
+        activityAction->setText(i18n("Add Activity"));
+        activityAction->setIcon(KIcon("list-add"));
+        activityAction->setVisible(false);
+        activityAction->setEnabled(false);
+        connect(activityAction, SIGNAL(triggered()), this, SLOT(addContainment()));
+        activityAction->setShortcut(KShortcut("alt+d, alt+a"));
+        activityAction->setShortcutContext(Qt::ApplicationShortcut);
+
+        KAction *zoomAction = c->addAction("zoom out");
+        zoomAction->setText(i18n("Zoom Out"));
+        zoomAction->setIcon(KIcon("zoom-out"));
+        connect(zoomAction, SIGNAL(triggered()), this, SLOT(zoomOut()));
+        zoomAction->setShortcut(KShortcut("alt+d, -"));
+
+        c->updateShortcuts();
+
+        //add stuff to shortcut config
+        c->addShortcuts(DesktopView::shortcutActions(this));
+
         m_corona = c;
         c->setItemIndexMethod(QGraphicsScene::NoIndex);
         c->initializeLayout();
@@ -531,7 +588,8 @@ void PlasmaApp::showAppletBrowser()
     }
 
     foreach (DesktopView *view, m_desktops) {
-        if (view->containment() == containment && view->isDashboardVisible()) {
+        if (view->isDashboardVisible() &&
+            (view->containment() == containment || view->dashboardContainment() == containment)) {
             // the dashboard will pick this one up!
             return;
         }
@@ -572,7 +630,7 @@ bool PlasmaApp::hasComposite()
 {
 //    return true;
 #ifdef Q_WS_X11
-    return colormap && KWindowSystem::compositingActive();
+    return KWindowSystem::compositingActive();
 #else
     return false;
 #endif
@@ -610,19 +668,15 @@ void PlasmaApp::createView(Plasma::Containment *containment)
 
     // find the mapping of View to Containment, if any,
     // so we can restore things on start.
-    KConfigGroup viewIds(KGlobal::config(), "ViewIds");
-    int id = viewIds.readEntry(QString::number(containment->id()), 0);
-
-    WId viewWindow = 0;
 
     if (isPanelContainment(containment)) {
-        PanelView *panelView = new PanelView(containment, id);
-        viewWindow = panelView->winId();
-        connect(panelView, SIGNAL(destroyed(QObject*)), this, SLOT(panelRemoved(QObject*)));
-        m_panels << panelView;
-        panelView->show();
+        m_panelsWaiting << containment;
+        connect(containment, SIGNAL(destroyed(QObject*)), this, SLOT(waitingPanelRemoved(QObject*)));
+        m_panelViewCreationTimer->start();
     } else if (containment->screen() > -1 &&
                containment->screen() < Kephal::ScreenUtils::numScreens()) {
+        KConfigGroup viewIds(KGlobal::config(), "ViewIds");
+        int id = viewIds.readEntry(QString::number(containment->id()), 0);
         DesktopView *view = viewForScreen(containment->screen(), containment->desktop());
         if (view) {
             kDebug() << "had a view for" << containment->screen() << containment->desktop();
@@ -635,31 +689,61 @@ void PlasmaApp::createView(Plasma::Containment *containment)
 
         // we have a new screen. neat.
         view = new DesktopView(containment, id, 0);
-        viewWindow = view->winId();
         if (m_corona) {
             connect(m_corona, SIGNAL(screenOwnerChanged(int,int,Plasma::Containment*)),
                     view, SLOT(screenOwnerChanged(int,int,Plasma::Containment*)));
+            connect(m_corona, SIGNAL(shortcutsChanged()),
+                    view, SLOT(updateShortcuts()));
         }
 
         m_desktops.append(view);
         view->show();
+        setWmClass(view->winId());
     }
+}
 
+void PlasmaApp::setWmClass(WId id)
+{
 #ifdef Q_WS_X11
     //FIXME: if argb visuals enabled Qt will always set WM_CLASS as "qt-subapplication" no matter what
     //the application name is we set the proper XClassHint here, hopefully won't be necessary anymore when
     //qapplication will manage apps with argvisuals in a better way
-    if (viewWindow) {
-        XClassHint classHint;
-        classHint.res_name = const_cast<char*>("Plasma");
-        classHint.res_class = const_cast<char*>("Plasma");
-        XSetClassHint(QX11Info::display(), viewWindow, &classHint);
-    }
+    XClassHint classHint;
+    classHint.res_name = const_cast<char*>("Plasma");
+    classHint.res_class = const_cast<char*>("Plasma");
+    XSetClassHint(QX11Info::display(), id, &classHint);
 #endif
 }
 
+void PlasmaApp::createWaitingPanels()
+{
+    foreach (Plasma::Containment *containment, m_panelsWaiting) {
+        disconnect(containment, SIGNAL(destroyed(QObject*)), this, SLOT(waitingPanelRemoved(QObject*)));
+        KConfigGroup viewIds(KGlobal::config(), "ViewIds");
+        int id = viewIds.readEntry(QString::number(containment->id()), 0);
+        if (containment->screen() < Kephal::ScreenUtils::numScreens()) {
+            PanelView *panelView = new PanelView(containment, id);
+            connect(panelView, SIGNAL(destroyed(QObject*)), this, SLOT(panelRemoved(QObject*)));
+            m_panels << panelView;
+            panelView->show();
+            setWmClass(panelView->winId());
+        }
+    }
+
+    m_panelsWaiting.clear();
+}
+
 void PlasmaApp::containmentAdded(Plasma::Containment *containment)
 {
+    if (isPanelContainment(containment)) {
+        foreach (PanelView * panel, m_panels) {
+            if (panel->containment() == containment) {
+                kDebug() << "not creating second PanelView with existing Containment!!";
+                return;
+            }
+        }
+    }
+
     createView(containment);
     disconnect(containment, 0, this, 0);
     connect(containment, SIGNAL(zoomRequested(Plasma::Containment*,Plasma::ZoomDirection)),
@@ -668,18 +752,21 @@ void PlasmaApp::containmentAdded(Plasma::Containment *containment)
     connect(containment, SIGNAL(configureRequested(Plasma::Containment*)),
             this, SLOT(configureContainment(Plasma::Containment*)));
 
-    if (!isPanelContainment(containment)) {
-        connect(containment, SIGNAL(addSiblingContainment(Plasma::Containment *)),
-                this, SLOT(addContainment(Plasma::Containment *)));
+    if (containment->containmentType() == Plasma::Containment::DesktopContainment
+            && m_zoomLevel == Plasma::DesktopZoom) {
+        foreach (QAction *action, m_corona->actions()) {
+            containment->addToolBoxAction(action);
+        }
     }
 }
 
 void PlasmaApp::configureContainment(Plasma::Containment *containment)
 {
-    BackgroundDialog *configDialog = 0;
+    const QString id = "plasma_containment_settings_" + QString::number(containment->id());
+    BackgroundDialog *configDialog = qobject_cast<BackgroundDialog*>(KConfigDialog::exists(id));
+    kDebug() << configDialog;
 
-    if (m_configDialogs.contains(containment)) {
-        configDialog = m_configDialogs.value(containment);
+    if (configDialog) {
         configDialog->reloadConfig();
     } else {
         const QSize resolution = QApplication::desktop()->screenGeometry(containment->screen()).size();
@@ -698,10 +785,11 @@ void PlasmaApp::configureContainment(Plasma::Containment *containment)
 
         }
 
-        configDialog = new BackgroundDialog(resolution, containment, view);
+        KConfigSkeleton *nullManager = new KConfigSkeleton(0);
+        configDialog = new BackgroundDialog(resolution, containment, view, 0, id, nullManager);
         configDialog->setAttribute(Qt::WA_DeleteOnClose);
-        connect(configDialog, SIGNAL(destroyed(QObject*)),
-                this, SLOT(configDialogRemoved(QObject*)));
+
+        connect(configDialog, SIGNAL(destroyed(QObject*)), nullManager, SLOT(deleteLater()));
     }
 
     configDialog->show();
@@ -709,14 +797,22 @@ void PlasmaApp::configureContainment(Plasma::Containment *containment)
     KWindowSystem::activateWindow(configDialog->winId());
 }
 
-void PlasmaApp::addContainment(Plasma::Containment *fromContainment)
+void PlasmaApp::addContainment()
 {
+    //try to find the "active" containment to get a plugin name
+    int currentScreen = Kephal::ScreenUtils::screenId(QCursor::pos());
+    int currentDesktop = -1;
+    if (AppSettings::perVirtualDesktopViews()) {
+        currentDesktop = KWindowSystem::currentDesktop()-1;
+    }
+    Plasma::Containment *fromContainment=m_corona->containmentForScreen(currentScreen, currentDesktop);
+
     QString plugin = fromContainment ? fromContainment->pluginName() : QString();
     Plasma::Containment *c = m_corona->addContainment(plugin);
 
     if (c && fromContainment) {
         foreach (DesktopView *view, m_desktops) {
-            if (view->containment() == c){
+            if (view->containment() == fromContainment){
                 view->setContainment(c);
                 return;
             }
@@ -728,6 +824,11 @@ void PlasmaApp::addContainment(Plasma::Containment *fromContainment)
     }
 }
 
+void PlasmaApp::zoomOut()
+{
+    zoom(0, Plasma::ZoomOut);
+}
+
 void PlasmaApp::zoom(Plasma::Containment *containment, Plasma::ZoomDirection direction)
 {
     if (direction == Plasma::ZoomIn) {
@@ -739,7 +840,7 @@ void PlasmaApp::zoom(Plasma::Containment *containment, Plasma::ZoomDirection dir
         if (m_zoomLevel == Plasma::DesktopZoom) {
             int currentDesktop = -1;
             if (AppSettings::perVirtualDesktopViews()) {
-                currentDesktop = KWindowSystem::currentDesktop();
+                currentDesktop = KWindowSystem::currentDesktop()-1;
             }
 
             DesktopView *view = viewForScreen(desktop()->screenNumber(QCursor::pos()), currentDesktop);
@@ -763,15 +864,14 @@ void PlasmaApp::zoomIn(Plasma::Containment *containment)
     bool zoomIn = true;
     bool zoomOut = true;
     bool addSibling = isMutable;
-    bool lock = false;
     bool remove = false;
 
     if (m_zoomLevel == Plasma::GroupZoom) {
+        setControllerVisible(false);
         m_zoomLevel = Plasma::DesktopZoom;
         containment->closeToolBox();
         addSibling = false;
         zoomIn = false;
-        lock = true;
     } else if (m_zoomLevel == Plasma::OverviewZoom) {
         m_zoomLevel = Plasma::GroupZoom;
         remove = isMutable && true;
@@ -783,13 +883,18 @@ void PlasmaApp::zoomIn(Plasma::Containment *containment)
             continue;
         }
 
+        if (m_zoomLevel == Plasma::DesktopZoom) {
+            foreach (QAction *action, m_corona->actions()) {
+                c->addToolBoxAction(action);
+            }
+        }
+
         c->enableAction("zoom in", zoomIn);
-        c->enableAction("zoom out", zoomOut);
-        c->enableAction("add sibling containment", addSibling);
-        c->enableAction("lock widgets", lock);
         c->enableAction("remove", remove && (c->screen() == -1));
         c->enableAction("add widgets", isMutable);
     }
+    m_corona->enableAction("zoom out", zoomOut);
+    m_corona->enableAction("add sibling containment", addSibling);
 }
 
 void PlasmaApp::zoomOut(Plasma::Containment *)
@@ -797,11 +902,11 @@ void PlasmaApp::zoomOut(Plasma::Containment *)
     bool isMutable = m_corona->immutability() == Plasma::Mutable;
     bool zoomIn = true;
     bool zoomOut = true;
-    bool addSibling = isMutable && true;
-    bool lock = false;
+    bool addSibling = isMutable && true; //FIXME wtf?
     bool addWidgets = isMutable && true;
 
     if (m_zoomLevel == Plasma::DesktopZoom) {
+        setControllerVisible(true);
         m_zoomLevel = Plasma::GroupZoom;
     } else if (m_zoomLevel == Plasma::GroupZoom) {
         m_zoomLevel = Plasma::OverviewZoom;
@@ -815,29 +920,177 @@ void PlasmaApp::zoomOut(Plasma::Containment *)
             continue;
         }
 
+        if (m_zoomLevel == Plasma::GroupZoom) {
+            foreach (QAction *action, m_corona->actions()) {
+                c->removeToolBoxAction(action);
+            }
+        }
+
         c->enableAction("zoom in", zoomIn);
-        c->enableAction("zoom out", zoomOut);
-        c->enableAction("add sibling containment", addSibling);
-        c->enableAction("lock widgets", lock);
         c->enableAction("remove", isMutable && c->screen() == -1);
         c->enableAction("add widgets", addWidgets);
     }
+    m_corona->enableAction("zoom out", zoomOut);
+    m_corona->enableAction("add sibling containment", addSibling);
 }
 
-void PlasmaApp::panelRemoved(QObject* panel)
+void PlasmaApp::setControllerVisible(bool show)
 {
-    m_panels.removeAll((PanelView*)panel);
-}
+    if (show) {
+        if (!m_controllerDialog) {
+            m_controllerDialog = new Plasma::Dialog;
+            QVBoxLayout *layout = new QVBoxLayout(m_controllerDialog);
 
-void PlasmaApp::configDialogRemoved(QObject* dialog)
-{
-    QMutableHashIterator<Plasma::Containment *, BackgroundDialog *> it(m_configDialogs);
-    while (it.hasNext()) {
-        it.next();
-        if (it.value() == (BackgroundDialog*)dialog) {
-           it.remove();
+            foreach (QAction *action, m_corona->actions()) {
+                ToolButton *actionButton = new ToolButton(m_controllerDialog);
+                actionButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
+                actionButton->setDefaultAction(action);
+                layout->addWidget(actionButton);
+            }
+
+            ToolButton *actionButton = new ToolButton(m_controllerDialog);
+            actionButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
+            actionButton->setIcon(KIcon("configure"));
+            actionButton->setText(i18n("Configure Plasma..."));
+            layout->addWidget(actionButton);
+            connect(actionButton, SIGNAL(clicked()), this, SLOT(createConfigurationInterface()));
         }
+
+        m_controllerDialog->show();
+    } else if (!show) {
+        delete m_controllerDialog;
+        m_controllerDialog = 0;
     }
 }
 
+void PlasmaApp::createConfigurationInterface()
+{
+    QWidget *widget = new QWidget();
+    m_configUi.setupUi(widget);
+    KConfigSkeleton *nullManager = new KConfigSkeleton(0);
+    KConfigDialog *dialog = new KConfigDialog(0, "Plasma settings", nullManager);
+    dialog ->addPage(widget, i18n("Plasma settings"));
+    dialog->setAttribute(Qt::WA_DeleteOnClose, true);
+    dialog->setFaceType(KPageDialog::Auto);
+    dialog->showButton(KDialog::Apply, false);
+
+    //try to find out if every view has a DashboardContainment
+    bool dashboardFollowsDesktop = true;
+
+    foreach (DesktopView *view, m_desktops) {
+        if (view->dashboardContainment()) {
+            dashboardFollowsDesktop = false;
+            break;
+        }
+    }
+
+    m_configUi.fixedDashboard->setChecked(!dashboardFollowsDesktop);
+    m_configUi.perVirtualDesktopViews->setChecked(AppSettings::perVirtualDesktopViews());
+
+    connect(dialog, SIGNAL(okClicked()), this, SLOT(configAccepted()));
+
+    dialog->show();
+}
+
+void PlasmaApp::configAccepted()
+{
+    setPerVirtualDesktopViews(m_configUi.perVirtualDesktopViews->isChecked());
+    setFixedDashboard(m_configUi.fixedDashboard->isChecked());
+}
+
+void PlasmaApp::setPerVirtualDesktopViews(bool perDesktopViews)
+{
+    AppSettings::setPerVirtualDesktopViews(perDesktopViews);
+    AppSettings::self()->writeConfig();
+
+    disconnect(KWindowSystem::self(), SIGNAL(numberOfDesktopsChanged(int)),
+               this, SLOT(checkVirtualDesktopViews(int)));
+
+    if (perDesktopViews) {
+        connect(KWindowSystem::self(), SIGNAL(numberOfDesktopsChanged(int)),
+                this, SLOT(checkVirtualDesktopViews(int)));
+        checkVirtualDesktopViews(KWindowSystem::numberOfDesktops());
+    } else {
+        int currentDesktop = KWindowSystem::currentDesktop();
+        QList<DesktopView *> perScreenViews;
+        foreach (DesktopView *view, m_desktops) {
+            if (view->containment()) {
+                view->containment()->setScreen(-1, -1);
+            }
+
+            delete view;
+        }
+
+        m_desktops.clear();
+        m_corona->checkScreens(true);
+    }
+
+    foreach (DesktopView *view, m_desktops) {
+        view->zoomOut(m_zoomLevel);
+    }
+
+    foreach (Plasma::Containment *c, m_corona->containments()) {
+        c->enableAction("zoom in", true);
+    }
+}
+
+void PlasmaApp::checkVirtualDesktopViews(int numDesktops)
+{
+    kDebug() << numDesktops;
+    if (AppSettings::perVirtualDesktopViews()) {
+        QMutableListIterator<DesktopView *> it(m_desktops);
+        while (it.hasNext()) {
+            DesktopView *view = it.next();
+            if (!view->containment() || view->desktop() >= numDesktops)  {
+                delete view;
+                it.remove();
+            }
+        }
+    }
+
+    m_corona->checkScreens(true);
+}
+
+void PlasmaApp::setFixedDashboard(bool fixedDashboard)
+{
+    bool dashboardFollowsDesktop = true;
+    foreach (DesktopView *view, m_desktops) {
+        if (view->dashboardContainment()) {
+            dashboardFollowsDesktop = false;
+            break;
+        }
+    }
+
+    Plasma::Containment *c = 0;
+    if (fixedDashboard) {
+        //avoid the containmentAdded signal being emitted
+        m_corona->blockSignals(true);
+        c = m_corona->addContainment("desktop");
+        m_corona->blockSignals(false);
+        m_corona->addOffscreenWidget(c);
+    }
+
+    foreach (DesktopView *view, m_desktops) {
+        view->setDashboardContainment(c);
+    }
+}
+
+void PlasmaApp::panelRemoved(QObject *panel)
+{
+    m_panels.removeAll((PanelView *)panel);
+}
+
+void PlasmaApp::waitingPanelRemoved(QObject *panelContainment)
+{
+    m_panelsWaiting.removeAll((Plasma::Containment *)panelContainment);
+}
+
+void PlasmaApp::updateActions(Plasma::ImmutabilityType immutability)
+{
+    bool enable = immutability == Plasma::Mutable && m_zoomLevel != Plasma::DesktopZoom;
+    kDebug() << enable;
+    m_corona->enableAction("add sibling containment", enable);
+}
+
+
 #include "plasmaapp.moc"
